@@ -1,7 +1,7 @@
 /* mpfr_get_f -- convert a MPFR number to a GNU MPF number
 
-Copyright 2005-2022 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 2005-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,9 +16,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #define MPFR_NEED_MPF_INTERNALS
 #include "mpfr-impl.h"
@@ -49,8 +48,8 @@ mpfr_get_f (mpf_ptr x, mpfr_srcptr y, mpfr_rnd_t rnd_mode)
           MPFR_SET_ERANGEFLAG ();
           return 0;
         }
-      else /* y is plus infinity (resp. minus infinity), set x to the maximum
-              value (resp. the minimum value) in precision PREC(x) */
+      else /* y is +inf (resp. -inf); set x to the maximum value
+              (resp. the minimum value) in precision PREC(x) */
         {
           int i;
           mp_limb_t *xp;
