@@ -1,7 +1,7 @@
 /* mpfr_set_exp - set the exponent of a floating-point number
 
-Copyright 2002-2004, 2006-2015 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 2002-2004, 2006-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,16 +16,17 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include "mpfr-impl.h"
 
 int
 mpfr_set_exp (mpfr_ptr x, mpfr_exp_t exponent)
 {
-  if (exponent >= __gmpfr_emin && exponent <= __gmpfr_emax)
+  if (MPFR_LIKELY (MPFR_IS_PURE_FP (x) &&
+                   exponent >= __gmpfr_emin &&
+                   exponent <= __gmpfr_emax))
     {
       MPFR_EXP(x) = exponent; /* do not use MPFR_SET_EXP of course... */
       return 0;
