@@ -1,7 +1,7 @@
 /* __gmpfr_floor_log2 - returns floor(log(d)/log(2))
 
-Copyright 1999-2004, 2006-2017 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 1999-2004, 2006-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,9 +16,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include "mpfr-impl.h"
 
@@ -26,28 +25,32 @@ http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 long
 __gmpfr_floor_log2 (double d)
 {
-#if _GMP_IEEE_FLOATS
-  union ieee_double_extract x;
+  long exp;
+#if _MPFR_IEEE_FLOATS
+  union mpfr_ieee_double_extract x;
 
   x.d = d;
-  return (long) x.s.exp - 1023;
-#else
-  long exp;
+  /* The cast below is useless in theory, but let us not depend on the
+     integer promotion rules (for instance, tcc is currently wrong). */
+  exp = (long) x.s.exp - 1023;
+  MPFR_ASSERTN (exp < 1023);  /* fail on infinities */
+  return exp;
+#else /* _MPFR_IEEE_FLOATS */
   double m;
 
   MPFR_ASSERTD (d >= 0);
   if (d >= 1.0)
     {
       exp = -1;
-      for( m= 1.0 ; m <= d ; m *=2.0 )
+      for (m = 1.0; m <= d; m *= 2.0)
         exp++;
     }
   else
     {
       exp = 0;
-      for( m= 1.0 ; m > d ; m *= (1.0/2.0) )
+      for (m = 1.0; m > d; m *= 0.5)
         exp--;
     }
   return exp;
-#endif
+#endif /* _MPFR_IEEE_FLOATS */
 }
