@@ -1,7 +1,7 @@
 /* Test file for mpfr_d_div
 
-Copyright 2007-2015 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 2007-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,17 +16,13 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <float.h>
 
 #include "mpfr-test.h"
-
-#if MPFR_VERSION >= MPFR_VERSION_NUM(2,4,0)
+#include "ieee_floats.h"
 
 static void
 check_nans (void)
@@ -106,7 +102,7 @@ check_nans (void)
   /* +inf / 0 == +inf */
   mpfr_set_ui (x, 0, MPFR_RNDN);
   mpfr_clear_flags ();
-  inexact = mpfr_d_div (y, DBL_POS_INF, x, MPFR_RNDN);
+  inexact = mpfr_d_div (y, MPFR_DBL_INFP, x, MPFR_RNDN);
   MPFR_ASSERTN (inexact == 0);
   MPFR_ASSERTN (__gmpfr_flags == 0);
   MPFR_ASSERTN (mpfr_inf_p (y));
@@ -115,7 +111,7 @@ check_nans (void)
   /* -inf / 0 == -inf */
   mpfr_set_ui (x, 0, MPFR_RNDN);
   mpfr_clear_flags ();
-  inexact = mpfr_d_div (y, DBL_NEG_INF, x, MPFR_RNDN);
+  inexact = mpfr_d_div (y, MPFR_DBL_INFM, x, MPFR_RNDN);
   MPFR_ASSERTN (inexact == 0);
   MPFR_ASSERTN (__gmpfr_flags == 0);
   MPFR_ASSERTN (mpfr_inf_p (y));
@@ -125,7 +121,7 @@ check_nans (void)
   mpfr_set_ui (x, 0, MPFR_RNDN);
   mpfr_neg (x, x, MPFR_RNDN);
   mpfr_clear_flags ();
-  inexact = mpfr_d_div (y, DBL_POS_INF, x, MPFR_RNDN);
+  inexact = mpfr_d_div (y, MPFR_DBL_INFP, x, MPFR_RNDN);
   MPFR_ASSERTN (inexact == 0);
   MPFR_ASSERTN (__gmpfr_flags == 0);
   MPFR_ASSERTN (mpfr_inf_p (y));
@@ -135,7 +131,7 @@ check_nans (void)
   mpfr_set_ui (x, 0, MPFR_RNDN);
   mpfr_neg (x, x, MPFR_RNDN);
   mpfr_clear_flags ();
-  inexact = mpfr_d_div (y, DBL_NEG_INF, x, MPFR_RNDN);
+  inexact = mpfr_d_div (y, MPFR_DBL_INFM, x, MPFR_RNDN);
   MPFR_ASSERTN (inexact == 0);
   MPFR_ASSERTN (__gmpfr_flags == 0);
   MPFR_ASSERTN (mpfr_inf_p (y));
@@ -191,19 +187,8 @@ main (void)
 
   check_nans ();
 
-  test_generic (2, 1000, 100);
+  test_generic (MPFR_PREC_MIN, 1000, 100);
 
   tests_end_mpfr ();
   return 0;
 }
-
-#else
-
-int
-main (void)
-{
-  printf ("Warning! Test disabled for this MPFR version.\n");
-  return 0;
-}
-
-#endif
