@@ -1,7 +1,7 @@
 /* Test file for mpfr_div (and some mpfr_div_ui, etc. tests).
 
-Copyright 1999, 2001-2017 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 1999, 2001-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,14 +16,13 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include "mpfr-test.h"
 
 static void
-check_equal (mpfr_srcptr a, mpfr_srcptr a2, char *s,
+check_equal (mpfr_srcptr a, mpfr_srcptr a2, const char *s,
              mpfr_srcptr b, mpfr_srcptr c, mpfr_rnd_t r)
 {
   if (SAME_VAL (a, a2))
@@ -69,8 +68,18 @@ mpfr_all_div (mpfr_ptr a, mpfr_srcptr b, mpfr_srcptr c, mpfr_rnd_t r)
         {
           __gmpfr_flags = oldflags;
           inex2 = mpfr_ui_div (a2, mpfr_get_ui (b, MPFR_RNDN), c, r);
-          MPFR_ASSERTN (SAME_SIGN (inex2, inex));
-          MPFR_ASSERTN (__gmpfr_flags == newflags);
+          if (!SAME_SIGN (inex2, inex))
+            {
+              printf ("Error for ternary value (rnd=%s), mpfr_div %d, mpfr_ui_div %d\n",
+                      mpfr_print_rnd_mode (r), inex, inex2);
+              exit (1);
+            }
+          if (__gmpfr_flags != newflags)
+            {
+              printf ("Error for flags, mpfr_div %d, mpfr_ui_div %d\n",
+                      newflags, __gmpfr_flags);
+              exit (1);
+            }
           check_equal (a, a2, "mpfr_ui_div", b, c, r);
         }
       if (mpfr_fits_slong_p (b, MPFR_RNDA))
@@ -359,7 +368,7 @@ check_convergence (void)
 /* given y = o(x/u), x, u, find the inexact flag by
    multiplying y by u */
 static int
-get_inexact (mpfr_t y, mpfr_t x, mpfr_t u)
+get_inexact (mpfr_ptr y, mpfr_ptr x, mpfr_ptr u)
 {
   mpfr_t xx;
   int inex;
@@ -913,7 +922,7 @@ check_special (void)
   set_emax (1);
   mpfr_set_ui (a, 1, MPFR_RNDZ);
   mpfr_set_ui (d, 1, MPFR_RNDZ);
-  mpfr_div_2exp (d, d, 1, MPFR_RNDZ);
+  mpfr_div_2ui (d, d, 1, MPFR_RNDZ);
   mpfr_clear_flags ();
   test_div (q, a, d, MPFR_RNDU); /* 1 / 0.5 = 2 -> overflow */
   MPFR_ASSERTN (mpfr_inf_p (q) && mpfr_sgn (q) > 0);
@@ -924,7 +933,7 @@ check_special (void)
   emin = mpfr_get_emin ();
   set_emin (-1);
   mpfr_set_ui (a, 1, MPFR_RNDZ);
-  mpfr_div_2exp (a, a, 2, MPFR_RNDZ);
+  mpfr_div_2ui (a, a, 2, MPFR_RNDZ);
   mpfr_set_prec (d, mpfr_get_prec (q) + 8);
   for (i = -1; i <= 1; i++)
     {
@@ -1072,12 +1081,7 @@ test_20070628 (void)
   int inex, err = 0;
 
   old_emax = mpfr_get_emax ();
-
-  if (mpfr_set_emax (256))
-    {
-      printf ("Can't change exponent range\n");
-      exit (1);
-    }
+  set_emax (256);
 
   mpfr_inits2 (53, x, y, (mpfr_ptr) 0);
   mpfr_set_si (x, -1, MPFR_RNDN);
@@ -1101,7 +1105,7 @@ test_20070628 (void)
       err++;
     }
   mpfr_clears (x, y, (mpfr_ptr) 0);
-  mpfr_set_emax (old_emax);
+  set_emax (old_emax);
 }
 
 /* Bug in mpfr_divhigh_n_basecase when all limbs of q (except the most
@@ -1128,9 +1132,13 @@ test_20151023 (void)
       mpfr_init2 (q0, GMP_NUMB_BITS);
 
       /* generate a random divisor of p bits */
-      mpfr_urandomb (d, RANDS);
-      /* generate a random quotient of GMP_NUMB_BITS bits */
-      mpfr_urandomb (q0, RANDS);
+      do
+        mpfr_urandomb (d, RANDS);
+      while (mpfr_zero_p (d));
+      /* generate a random non-zero quotient of GMP_NUMB_BITS bits */
+      do
+        mpfr_urandomb (q0, RANDS);
+      while (mpfr_zero_p (q0));
       /* zero-pad the quotient to p bits */
       inex = mpfr_prec_round (q0, p, MPFR_RNDN);
       MPFR_ASSERTN(inex == 0);
@@ -1152,7 +1160,16 @@ test_20151023 (void)
           MPFR_ASSERTN(inex == 0);
           mpfr_nextabove (n);
           mpfr_div (q, n, d, MPFR_RNDN);
-          MPFR_ASSERTN(mpfr_cmp (q, q0) == 0);
+          if (! mpfr_equal_p (q, q0))
+            {
+              printf ("Error in test_20151023 for p=%ld, rnd=RNDN, i=%d\n",
+                      (long) p, i);
+              printf ("n="); mpfr_dump (n);
+              printf ("d="); mpfr_dump (d);
+              printf ("expected q0="); mpfr_dump (q0);
+              printf ("got       q="); mpfr_dump (q);
+              exit (1);
+            }
 
           inex = mpfr_mul (n, d, q0, MPFR_RNDN);
           MPFR_ASSERTN(inex == 0);
@@ -1234,8 +1251,8 @@ test_extreme (void)
   emin = mpfr_get_emin ();
   emax = mpfr_get_emax ();
 
-  mpfr_set_emin (MPFR_EMIN_MIN);
-  mpfr_set_emax (MPFR_EMAX_MAX);
+  set_emin (MPFR_EMIN_MIN);
+  set_emax (MPFR_EMAX_MAX);
 
   for (xi = 0; xi < 4; xi++)
     {
@@ -1376,7 +1393,7 @@ test_mpfr_divsp2 (void)
   mpfr_set_str (v, "241810647971575979588130185988987264768", 10, MPFR_RNDN);
   mpfr_div (q, u, v, MPFR_RNDN);
   mpfr_set_str (u, "5732952910203749289426944", 10, MPFR_RNDN);
-  mpfr_div_2exp (u, u, 82, MPFR_RNDN);
+  mpfr_div_2ui (u, u, 82, MPFR_RNDN);
   MPFR_ASSERTN(mpfr_equal_p (q, u));
 
   mpfr_clear (u);
@@ -1405,7 +1422,7 @@ test_20160831 (void)
   mpfr_set_str (v, "205987256581218236405412302590110119580", 10, MPFR_RNDN);
   mpfr_div (q, u, v, MPFR_RNDN);
   mpfr_set_str (u, "19217137613667309953639458782352244736", 10, MPFR_RNDN);
-  mpfr_div_2exp (u, u, 124, MPFR_RNDN);
+  mpfr_div_2ui (u, u, 124, MPFR_RNDN);
   MPFR_ASSERTN (mpfr_equal_p (q, u));
 
   mpfr_clears (u, v, q, (mpfr_ptr) 0);
@@ -1519,11 +1536,212 @@ bug20171218 (void)
   mpfr_clear (c);
 }
 
+/* Extended test based on a bug found with flint-arb test suite with a
+   32-bit ABI: https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=888459
+   Division of the form: (1 - 2^(-pa)) / (1 - 2^(-pb)).
+   The result is compared to the one obtained by increasing the precision of
+   the divisor (without changing its value, so that both results should be
+   equal). For all of these tests, a failure may occur in r12126 only with
+   pb=GMP_NUMB_BITS and MPFR_RNDN (and some particular values of pa and pc).
+   This bug was introduced by r9086, where mpfr_div uses mpfr_div_ui when
+   the divisor has only one limb.
+*/
+static void
+bug20180126 (void)
+{
+  mpfr_t a, b1, b2, c1, c2;
+  int pa, i, j, pc, sa, sb, r, inex1, inex2;
+
+  for (pa = 100; pa < 800; pa += 11)
+    for (i = 1; i <= 4; i++)
+      for (j = -2; j <= 2; j++)
+        {
+          int pb = GMP_NUMB_BITS * i + j;
+
+          if (pb > pa)
+            continue;
+
+          mpfr_inits2 (pa, a, b1, (mpfr_ptr) 0);
+          mpfr_inits2 (pb, b2, (mpfr_ptr) 0);
+
+          mpfr_set_ui (a, 1, MPFR_RNDN);
+          mpfr_nextbelow (a);                   /* 1 - 2^(-pa) */
+          mpfr_set_ui (b2, 1, MPFR_RNDN);
+          mpfr_nextbelow (b2);                  /* 1 - 2^(-pb) */
+          inex1 = mpfr_set (b1, b2, MPFR_RNDN);
+          MPFR_ASSERTN (inex1 == 0);
+
+          for (pc = 32; pc <= 320; pc += 32)
+            {
+              mpfr_inits2 (pc, c1, c2, (mpfr_ptr) 0);
+
+              for (sa = 0; sa < 2; sa++)
+                {
+                  for (sb = 0; sb < 2; sb++)
+                    {
+                      RND_LOOP_NO_RNDF (r)
+                        {
+                          MPFR_ASSERTN (mpfr_equal_p (b1, b2));
+                          inex1 = mpfr_div (c1, a, b1, (mpfr_rnd_t) r);
+                          inex2 = mpfr_div (c2, a, b2, (mpfr_rnd_t) r);
+
+                          if (! mpfr_equal_p (c1, c2) ||
+                              ! SAME_SIGN (inex1, inex2))
+                            {
+                              printf ("Error in bug20180126 for "
+                                      "pa=%d pb=%d pc=%d sa=%d sb=%d %s\n",
+                                      pa, pb, pc, sa, sb,
+                                      mpfr_print_rnd_mode ((mpfr_rnd_t) r));
+                              printf ("inex1 = %d, c1 = ", inex1);
+                              mpfr_dump (c1);
+                              printf ("inex2 = %d, c2 = ", inex2);
+                              mpfr_dump (c2);
+                              exit (1);
+                            }
+                        }
+
+                      mpfr_neg (b1, b1, MPFR_RNDN);
+                      mpfr_neg (b2, b2, MPFR_RNDN);
+                    }  /* sb */
+
+                  mpfr_neg (a, a, MPFR_RNDN);
+                }  /* sa */
+
+              mpfr_clears (c1, c2, (mpfr_ptr) 0);
+            }  /* pc */
+
+          mpfr_clears (a, b1, b2, (mpfr_ptr) 0);
+        }  /* j */
+}
+
+static void
+coverage (mpfr_prec_t pmax)
+{
+  mpfr_prec_t p;
+
+  for (p = MPFR_PREC_MIN; p <= pmax; p++)
+    {
+      int inex;
+      mpfr_t q, u, v;
+
+      mpfr_init2 (q, p);
+      mpfr_init2 (u, p);
+      mpfr_init2 (v, p);
+
+      /* exercise case qx < emin */
+      mpfr_set_ui_2exp (u, 1, mpfr_get_emin (), MPFR_RNDN);
+      mpfr_set_ui (v, 4, MPFR_RNDN);
+
+      mpfr_clear_flags ();
+      /* u/v = 2^(emin-2), should be rounded to +0 for RNDN */
+      inex = mpfr_div (q, u, v, MPFR_RNDN);
+      MPFR_ASSERTN(inex < 0);
+      MPFR_ASSERTN(mpfr_zero_p (q) && mpfr_signbit (q) == 0);
+      MPFR_ASSERTN(mpfr_underflow_p ());
+
+      mpfr_clear_flags ();
+      /* u/v = 2^(emin-2), should be rounded to 2^(emin-1) for RNDU */
+      inex = mpfr_div (q, u, v, MPFR_RNDU);
+      MPFR_ASSERTN(inex > 0);
+      MPFR_ASSERTN(mpfr_cmp_ui_2exp (q, 1, mpfr_get_emin () - 1) == 0);
+      MPFR_ASSERTN(mpfr_underflow_p ());
+
+      mpfr_clear_flags ();
+      /* u/v = 2^(emin-2), should be rounded to +0 for RNDZ */
+      inex = mpfr_div (q, u, v, MPFR_RNDZ);
+      MPFR_ASSERTN(inex < 0);
+      MPFR_ASSERTN(mpfr_zero_p (q) && mpfr_signbit (q) == 0);
+      MPFR_ASSERTN(mpfr_underflow_p ());
+
+      if (p == 1)
+        goto end_of_loop;
+
+      mpfr_set_ui_2exp (u, 1, mpfr_get_emin (), MPFR_RNDN);
+      mpfr_nextbelow (u); /* u = (1-2^(-p))*2^emin */
+      mpfr_set_ui (v, 2, MPFR_RNDN);
+
+      mpfr_clear_flags ();
+      /* u/v = (1-2^(-p))*2^(emin-1), will round to 2^(emin-1) for RNDN */
+      inex = mpfr_div (q, u, v, MPFR_RNDN);
+      MPFR_ASSERTN(inex > 0);
+      MPFR_ASSERTN(mpfr_cmp_ui_2exp (q, 1, mpfr_get_emin () - 1) == 0);
+      MPFR_ASSERTN(mpfr_underflow_p ());
+
+      mpfr_clear_flags ();
+      /* u/v should round to 2^(emin-1) for RNDU */
+      inex = mpfr_div (q, u, v, MPFR_RNDU);
+      MPFR_ASSERTN(inex > 0);
+      MPFR_ASSERTN(mpfr_cmp_ui_2exp (q, 1, mpfr_get_emin () - 1) == 0);
+      MPFR_ASSERTN(mpfr_underflow_p ());
+
+      mpfr_clear_flags ();
+      /* u/v should round to +0 for RNDZ */
+      inex = mpfr_div (q, u, v, MPFR_RNDZ);
+      MPFR_ASSERTN(inex < 0);
+      MPFR_ASSERTN(mpfr_zero_p (q) && mpfr_signbit (q) == 0);
+      MPFR_ASSERTN(mpfr_underflow_p ());
+
+    end_of_loop:
+      mpfr_clear (q);
+      mpfr_clear (u);
+      mpfr_clear (v);
+    }
+}
+
+/* coverage for case usize >= n + n in Mulders' algorithm */
+static void
+coverage2 (void)
+{
+  mpfr_prec_t p;
+  mpfr_t q, u, v, t, w;
+  int inex, inex2;
+
+  p = MPFR_DIV_THRESHOLD * GMP_NUMB_BITS;
+  mpfr_init2 (q, p);
+  mpfr_init2 (u, 2 * p + 3 * GMP_NUMB_BITS);
+  mpfr_init2 (v, p);
+  do mpfr_urandomb (u, RANDS); while (mpfr_zero_p (u));
+  do mpfr_urandomb (v, RANDS); while (mpfr_zero_p (v));
+  inex = mpfr_div (q, u, v, MPFR_RNDN);
+  mpfr_init2 (t, mpfr_get_prec (u));
+  mpfr_init2 (w, mpfr_get_prec (u));
+  inex2 = mpfr_mul (t, q, v, MPFR_RNDN);
+  MPFR_ASSERTN(inex2 == 0);
+  if (inex == 0) /* check q*v = u */
+    MPFR_ASSERTN(mpfr_equal_p (u, t));
+  else
+    {
+      if (inex > 0)
+        mpfr_nextbelow (q);
+      else
+        mpfr_nextabove (q);
+      inex2 = mpfr_mul (w, q, v, MPFR_RNDN);
+      MPFR_ASSERTN(inex2 == 0);
+      inex2 = mpfr_sub (t, t, u, MPFR_RNDN);
+      MPFR_ASSERTN(inex2 == 0);
+      inex2 = mpfr_sub (w, w, u, MPFR_RNDN);
+      MPFR_ASSERTN(inex2 == 0);
+      MPFR_ASSERTN(mpfr_cmpabs (t, w) <= 0);
+      if (mpfr_cmpabs (t, w) == 0) /* even rule: significand of q should now
+                                      be odd */
+        MPFR_ASSERTN(mpfr_min_prec (q) == mpfr_get_prec (q));
+    }
+
+  mpfr_clear (q);
+  mpfr_clear (u);
+  mpfr_clear (v);
+  mpfr_clear (t);
+  mpfr_clear (w);
+}
+
 int
 main (int argc, char *argv[])
 {
   tests_start_mpfr ();
 
+  coverage (1024);
+  coverage2 ();
+  bug20180126 ();
   bug20171218 ();
   testall_rndf (9);
   test_20170105 ();
