@@ -1,7 +1,7 @@
 /* Test file for mpfr_urandomb
 
-Copyright 1999-2004, 2006-2017 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 1999-2004, 2006-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,9 +16,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include "mpfr-test.h"
 
@@ -31,12 +30,9 @@ test_urandomb (long nbtests, mpfr_prec_t prec, int verbose)
   mpfr_exp_t emin;
 
   size_tab = (nbtests >= 1000 ? nbtests / 50 : 20);
-  tab = (int *) calloc (size_tab, sizeof(int));
-  if (tab == NULL)
-    {
-      fprintf (stderr, "trandom: can't allocate memory in test_urandomb\n");
-      exit (1);
-    }
+  tab = (int *) tests_allocate (size_tab * sizeof (int));
+  for (k = 0; k < size_tab; k++)
+    tab[k] = 0;
 
   mpfr_init2 (x, prec);
   xn = 1 + (prec - 1) / mp_bits_per_limb;
@@ -71,32 +67,30 @@ test_urandomb (long nbtests, mpfr_prec_t prec, int verbose)
   set_emin (emin);
 
   mpfr_clear (x);
-  if (!verbose)
+
+  if (verbose)
     {
-      free(tab);
-      return;
+      av /= nbtests;
+      var = (var / nbtests) - av * av;
+
+      th = (double) nbtests / size_tab;
+      printf ("Average = %.5f\nVariance = %.5f\n", av, var);
+      printf ("Repartition for urandomb. Each integer should be close to"
+              " %d.\n", (int) th);
+
+      for (k = 0; k < size_tab; k++)
+        {
+          chi2 += (tab[k] - th) * (tab[k] - th) / th;
+          printf ("%d ", tab[k]);
+          if (((unsigned int) (k+1) & 7) == 0)
+            printf ("\n");
+        }
+
+      printf ("\nChi2 statistics value (with %d degrees of freedom) :"
+              " %.5f\n\n", size_tab - 1, chi2);
     }
 
-  av /= nbtests;
-  var = (var / nbtests) - av * av;
-
-  th = (double)nbtests / size_tab;
-  printf("Average = %.5f\nVariance = %.5f\n", av, var);
-  printf("Repartition for urandomb. Each integer should be close to %d.\n",
-         (int)th);
-
-  for (k = 0; k < size_tab; k++)
-    {
-      chi2 += (tab[k] - th) * (tab[k] - th) / th;
-      printf("%d ", tab[k]);
-      if (((k+1) & 7) == 0)
-        printf("\n");
-    }
-
-  printf("\nChi2 statistics value (with %d degrees of freedom) : %.5f\n\n",
-         size_tab - 1, chi2);
-
-  free(tab);
+  tests_free (tab, size_tab * sizeof (int));
   return;
 }
 
@@ -159,7 +153,7 @@ main (int argc, char *argv[])
   nbtests = 10000;
   if (argc > 1)
     {
-      long a = atol(argv[1]);
+      long a = atol (argv[1]);
       if (a != 0)
         nbtests = a;
     }
@@ -167,7 +161,7 @@ main (int argc, char *argv[])
   if (argc <= 2)
     prec = 1000;
   else
-    prec = atol(argv[2]);
+    prec = atol (argv[2]);
 
   test_urandomb (nbtests, prec, verbose);
 
@@ -190,7 +184,7 @@ main (int argc, char *argv[])
   {
     gmp_randstate_t s;
     mpfr_t x;
-    char *str = "0.1010111100000000000000000000000000000000E-32";
+    const char *str = "0.1010111100000000000000000000000000000000E-32";
     int k;
 
     gmp_randinit_default (s);
