@@ -1,7 +1,7 @@
 /* Test file for mpfr_set.
 
-Copyright 2001-2018 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 2001-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,9 +16,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include "mpfr-test.h"
 
@@ -207,7 +206,7 @@ check_special (void)
 static void
 check_ternary_value (void)
 {
-  int p, q, rnd;
+  int k, p, q, rnd;
   int inexact, cmp;
   mpfr_t x, y;
 
@@ -217,37 +216,54 @@ check_ternary_value (void)
     {
       mpfr_set_prec (x, p);
       mpfr_urandomb (x, RANDS);
-      if (randlimb () % 2)
+      if (RAND_BOOL ())
         mpfr_neg (x, x, MPFR_RNDN);
       for (q=2; q<2*p; q++)
         {
           mpfr_set_prec (y, q);
-          for (rnd = 0; rnd < MPFR_RND_MAX; rnd++)
+          RND_LOOP (rnd)
             {
               if (rnd == MPFR_RNDF) /* the test below makes no sense */
                 continue;
-              inexact = mpfr_set (y, x, (mpfr_rnd_t) rnd);
-              cmp = mpfr_cmp (y, x);
-              if (((inexact == 0) && (cmp != 0)) ||
-                  ((inexact > 0) && (cmp <= 0)) ||
-                  ((inexact < 0) && (cmp >= 0)))
+              for (k = 0; k < 3; k++)
                 {
-                  printf ("Wrong ternary value in mpfr_set for %s: expected"
-                          " %d, got %d\n",
-                          mpfr_print_rnd_mode ((mpfr_rnd_t) rnd), cmp,
-                          inexact);
-                  exit (1);
-                }
-              /* Test mpfr_set function too */
-              inexact = (mpfr_set) (y, x, (mpfr_rnd_t) rnd);
-              cmp = mpfr_cmp (y, x);
-              if (((inexact == 0) && (cmp != 0)) ||
-                  ((inexact > 0) && (cmp <= 0)) ||
-                  ((inexact < 0) && (cmp >= 0)))
-                {
-                  printf ("Wrong ternary value in mpfr_set(2): expected %d,"
-                          " got %d\n", cmp, inexact);
-                  exit (1);
+                  int a = 0, b = 0, c = 0;
+
+                  switch (k)
+                    {
+                    case 0:
+                      inexact = mpfr_set (y, x, (mpfr_rnd_t) rnd);
+                      break;
+                    case 1:
+                      inexact = (mpfr_set) (y, x, (mpfr_rnd_t) rnd);
+                      break;
+                    case 2:
+#ifdef IGNORE_CPP_COMPAT
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wc++-compat"
+#endif
+                      inexact = mpfr_set ((a++, VOIDP_CAST(y)),
+                                          (b++, VOIDP_CAST(x)),
+                                          (c++, (mpfr_rnd_t) rnd));
+#ifdef IGNORE_CPP_COMPAT
+#pragma GCC diagnostic pop
+#endif
+                      MPFR_ASSERTN (a == 1);
+                      MPFR_ASSERTN (b == 1);
+                      MPFR_ASSERTN (c == 1);
+                      break;
+                    }
+                  cmp = mpfr_cmp (y, x);
+                  if (((inexact == 0) && (cmp != 0)) ||
+                      ((inexact > 0) && (cmp <= 0)) ||
+                      ((inexact < 0) && (cmp >= 0)))
+                    {
+                      printf ("Wrong ternary value in mpfr_set for %s (%d):"
+                              " expected %d, got %d\n",
+                              mpfr_print_rnd_mode ((mpfr_rnd_t) rnd),
+                              k, cmp, inexact);
+                      exit (1);
+                    }
                 }
             }
         }
@@ -256,6 +272,93 @@ check_ternary_value (void)
   mpfr_clear (y);
 }
 
+static void
+test_set_1_2 (void)
+{
+  mpfr_t u, v, zz, z;
+  int inex;
+
+  /* (8,16)-bit test */
+  mpfr_inits2 (16, u, v, zz, (mpfr_ptr) 0);
+  mpfr_init2 (z, 8);
+  mpfr_set_str_binary (u, "0.1100001100011010E-1");
+  mpfr_set_str_binary (v, "0.1100010101110010E0");
+  /* u + v = 1.0010011011111111 */
+  inex = mpfr_add (zz, u, v, MPFR_RNDN);
+  MPFR_ASSERTN(inex > 0);
+  mpfr_set_str_binary (u, "1.001001110000000");
+  MPFR_ASSERTN(mpfr_equal_p (zz, u));
+  inex = mpfr_set_1_2 (z, zz, MPFR_RNDN, inex);
+  /* we should have z = 1.0010011 and inex < 0 */
+  MPFR_ASSERTN(inex < 0);
+  mpfr_set_str_binary (u, "1.0010011");
+  MPFR_ASSERTN(mpfr_equal_p (z, u));
+  mpfr_clears (u, v, zz, z, (mpfr_ptr) 0);
+
+  /* (16,32)-bit test:
+   * take for v a random 32-bit number in [1/2,1), here 2859611790/2^32
+   * take for z a random 16-bit number in [1,2), less than 2*v,
+   with last bit 0, here we take z = 40900/2^15
+   * take u = z-v-1/2^16-1/2^32 */
+  mpfr_inits2 (32, u, v, zz, (mpfr_ptr) 0);
+  mpfr_init2 (z, 16);
+  mpfr_set_str_binary (u, "0.10010101000101001100100101110001");
+  mpfr_set_str_binary (v, "0.10101010011100100011011010001110");
+  /* u + v = 1.00111111100001101111111111111111 */
+  inex = mpfr_add (zz, u, v, MPFR_RNDN);
+  MPFR_ASSERTN(inex > 0);
+  mpfr_set_str_binary (u, "1.0011111110000111");
+  MPFR_ASSERTN(mpfr_equal_p (zz, u));
+  inex = mpfr_set_1_2 (z, zz, MPFR_RNDN, inex);
+  /* we should have z = 1.001111111000011 and inex < 0 */
+  MPFR_ASSERTN(inex < 0);
+  mpfr_set_str_binary (u, "1.001111111000011");
+  MPFR_ASSERTN(mpfr_equal_p (z, u));
+  mpfr_clears (u, v, zz, z, (mpfr_ptr) 0);
+
+  /* (32,64)-bit test:
+   * take for v a random 64-bit number in [1/2,1),
+     here v = 13687985014345662879/2^64
+   * take for z a random 32-bit number in [1,2), less than 2*v,
+     with last bit 0, here we take z = 2871078774/2^31
+   * take u = z-v-1/2^32-1/2^64 */
+  mpfr_inits2 (64, u, v, zz, (mpfr_ptr) 0);
+  mpfr_init2 (z, 32);
+  mpfr_set_str_binary (u, "0.10011000010011001110000100010001110010010000111001111110011");
+  mpfr_set_str_binary (v, "0.1011110111110101011111011101100100110110111100011000000110011111");
+  /* u + v = 1.0101011001000010010111101110101011111111111111111111111111111111 */
+  inex = mpfr_add (zz, u, v, MPFR_RNDN);
+  MPFR_ASSERTN(inex > 0);
+  mpfr_set_str_binary (u, "1.01010110010000100101111011101011");
+  MPFR_ASSERTN(mpfr_equal_p (zz, u));
+  inex = mpfr_set_1_2 (z, zz, MPFR_RNDN, inex);
+  /* we should have z = 1.0101011001000010010111101110101 and inex < 0 */
+  MPFR_ASSERTN(inex < 0);
+  mpfr_set_str_binary (u, "1.0101011001000010010111101110101");
+  MPFR_ASSERTN(mpfr_equal_p (z, u));
+  mpfr_clears (u, v, zz, z, (mpfr_ptr) 0);
+
+  /* (64,128)-bit test:
+   * take for v a random 128-bit number in [1/2,1),
+     here v = 322263811942091240216761391118876232409/2^128
+   * take for z a random 64-bit number in [1,2), less than 2*v,
+     with last bit 0, here we take z = 16440347967874738276/2^63
+   * take u = z-v-1/2^64-1/2^128 */
+  mpfr_inits2 (128, u, v, zz, (mpfr_ptr) 0);
+  mpfr_init2 (z, 64);
+  mpfr_set_str_binary (u, "0.1101010111011101111100100001011111111000010011011001000101111010110101101101011011100110101001010001101011011110101101010010011");
+  mpfr_set_str_binary (v, "0.11110010011100011100000010100110100010011010110010111111010011000010100100101001000110010101101011100101001000010100101011011001");
+  inex = mpfr_add (zz, u, v, MPFR_RNDN);
+  MPFR_ASSERTN(inex > 0);
+  mpfr_set_str_binary (u, "1.1100100001001111101100101011111010000001111110100101000011000111");
+  MPFR_ASSERTN(mpfr_equal_p (zz, u));
+  inex = mpfr_set_1_2 (z, zz, MPFR_RNDN, inex);
+  MPFR_ASSERTN(inex < 0);
+  mpfr_set_str_binary (u, "1.1100100001001111101100101011111010000001111110100101000011000110");
+  MPFR_ASSERTN(mpfr_equal_p (z, u));
+  mpfr_clears (u, v, zz, z, (mpfr_ptr) 0);
+}
+
 #define TEST_FUNCTION mpfr_set
 #include "tgeneric.c"
 
@@ -268,6 +371,8 @@ main (void)
 
   tests_start_mpfr ();
 
+  test_set_1_2 ();
+
   /* Default : no error */
   error = 0;
 
